//------------------------------------------------------------------------------
// File:   ondokei_DS18B20.c
// Created on 2020/11/19, 9:23
//------------------------------------------------------------------------------
#define p24FJ64GA002

#define FOSC 32000000   
#define FCY  (FOSC/2)

#define USE_AND_OR

#include <xc.h>
#include <stdio.h>
#include <stdbool.h>
#include <libpic30.h>
#include <uart.h>
#include <PPS.h>

#pragma config POSCMOD  = HS
#pragma config FNOSC    = PRIPLL
#pragma config OSCIOFNC = OFF
#pragma config FWDTEN   = OFF
#pragma config FCKSM    = CSDCMD

#define BAUD_RATE    9600
#define UxBRG_ARG  (((FCY/BAUD_RATE)/16)-1)   // = 103

#define PORT_INPUT  1
#define PORT_OUTPUT 0

#define OW_HIGH  1
#define OW_LOW   0

#define OW_SET    1
#define OW_CLEAR  0

#define OW_LAT   LATBbits.LATB3
#define OW_PORT  PORTBbits.RB3
#define OW_TRIS  TRISBbits.TRISB3

#define DS18B20_RESET_ERROR 0x0800

void drive_OW_high(void)
{
	OW_TRIS = PORT_OUTPUT;
	OW_LAT  = OW_HIGH;	
}

void drive_OW_low(void)
{
	OW_TRIS = PORT_OUTPUT;
	OW_LAT  = OW_LOW;
}

void drive_OW_input(void)
{
	OW_TRIS = PORT_INPUT;
}

unsigned char read_OW(void)
{
	unsigned char read_data;
	
	OW_TRIS = PORT_INPUT;

	if(OW_PORT == OW_HIGH) {
	 	read_data = OW_SET;
	} else { 	
		read_data = OW_CLEAR;
	}
	
	return read_data;
}

//------------------------------------------------------------------------------
//  return 0:connection error 1:sensor is present
//------------------------------------------------------------------------------
bool OW_Reset(void) {
	unsigned char presence_detect;
	
	drive_OW_low();
  __delay_us(480);
  drive_OW_input();
  __delay_us(480);		
	presence_detect = read_OW();
  __delay_us(410);		
	
  return presence_detect;
}

void OW_write_bit (unsigned char write_bit)
{
	if( write_bit ){	  //writing a bit '1'
		drive_OW_low(); 			// Drive the bus low
		__delay_us(6);				// delay 6 microsecond (us)
		drive_OW_high(); 			// Release the bus
		__delay_us(64);				// delay 64 microsecond (us)
	}	else {           //writing a bit '0'
		drive_OW_low(); 				// Drive the bus low
		__delay_us(60);					// delay 60 microsecond (us)
		drive_OW_high();  			// Release the bus
		__delay_us(10);					// delay 10 microsecond for recovery (us)
	}
}	

unsigned char OW_read_bit (void)
{
	unsigned char read_data; 
	//reading a bit 
	drive_OW_low(); 			 // Drive the bus low
	__delay_us(6)					 // delay 6 microsecond (us)
	drive_OW_high();  	   // Release the bus
	__delay_us(9)					 // delay 9 microsecond (us)

	read_data = read_OW(); //Read the status of OW_PIN

	__delay_us(55);				 // delay 55 microsecond (us)	
	return read_data;
}

void OW_write_byte(unsigned char write_data)
{
	unsigned char loop;
	
	for( loop = 0 ; loop < 8 ; loop++ )	{
		OW_write_bit(write_data & 0x01); 	//Sending LS-bit first
		write_data >>= 1;               	// shift the data byte for the next bit to send
	}	
}	

unsigned char OW_read_byte (void)
{
	unsigned char result = 0;
	unsigned char loop;
	
	for( loop = 0 ; loop < 8 ; loop++ ) {
		result >>= 1; 				// shift the result to get it ready for the next bit to receive
		if( OW_read_bit() ) {
		  result |= 0x80;				// if result is one, then set MS-bit
		}	
	}
	return result;					
}	

int ds18b20_read(void)
{
	uint16_t raw_temp_value;
	
  if(OW_Reset() == 0) {   // send start pulse
    return DS18B20_RESET_ERROR;           
  }

  OW_write_byte(0xCC);   // send skip ROM command
  OW_write_byte(0x44);   // send start conversion command
 
  while(OW_read_byte() == 0) { ; }  // wait for conversion complete
 
  if(OW_Reset() == 0) {   // send start pulse
    return DS18B20_RESET_ERROR;            
  }
	
  OW_write_byte(0xCC);  // send skip ROM command
  OW_write_byte(0xBE);  // send read command
  
  raw_temp_value  = OW_read_byte();         // read temperature LSB byte and store it on raw_temp_value LSB byte
  raw_temp_value |= (OW_read_byte() << 8);  // read temperature MSB byte and store it on raw_temp_value MSB byte
 
  return (int)raw_temp_value; 
}

int main(void) {
	int raw_tmperature;
	char buf[20];
	
  iPPSOutput( OUT_PIN_PPS_RP13, OUT_FN_PPS_U1TX );
  iPPSInput(  IN_FN_PPS_U1RX,  IN_PIN_PPS_RP14  ); 	
  OpenUART1( UART_EN, UART_TX_ENABLE, UxBRG_ARG );

	AD1PCFGbits.PCFG5 = 1; // 1=digital mode
					
	while( true ) {
		raw_tmperature = ds18b20_read();
		if( raw_tmperature == DS18B20_RESET_ERROR ) {
			sprintf(buf, "reset error\n" );
		} else {	
		  sprintf(buf, "%10.4f\n",  (double)raw_tmperature * 0.0625);
		}
		putsUART1( (unsigned int *)buf );					
	}    
}
